/*
 * Copyright (c) 2009 by Red Hat Inc and/or its affiliates or by
 * third-party contributors as indicated by either @author tags or express
 * copyright attribution statements applied by the authors.  All
 * third-party contributions are distributed under license by Red Hat Inc.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution; if not, write to:
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301  USA\
 */
package org.hibernate.engine.jdbc.jdbc3;

import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.sql.SQLException;
import java.sql.Blob;

/**
 * TODO : javadoc
 *
 * @author Steve Ebersole
 */
public class BlobImpl implements Blob {

	private InputStream stream;
	private long length;
	private boolean needsReset = false;

	public BlobImpl(byte[] bytes) {
		this.stream = new ByteArrayInputStream(bytes);
		this.length = bytes.length;
	}

	public BlobImpl(InputStream stream, long length) {
		this.stream = stream;
		this.length = length;
	}

	/**
	 * {@inheritDoc}
	 */
	public long length() throws SQLException {
		return length;
	}

	/**
	 * {@inheritDoc}
	 */
	public InputStream getBinaryStream() throws SQLException {
		try {
			if (needsReset) {
				stream.reset();
			}
		}
		catch ( IOException ioe) {
			throw new SQLException("could not reset reader");
		}
		needsReset = true;
		return stream;
	}

	/**
	 * {@inheritDoc}
	 */
	public void truncate(long pos) throws SQLException {
		throw excep();
	}

	/**
	 * {@inheritDoc}
	 */
	public byte[] getBytes(long pos, int len) throws SQLException {
		throw excep();
	}

	/**
	 * {@inheritDoc}
	 */
	public int setBytes(long pos, byte[] bytes) throws SQLException {
		throw excep();
	}

	/**
	 * {@inheritDoc}
	 */
	public int setBytes(long pos, byte[] bytes, int i, int j) throws SQLException {
		throw excep();
	}

	/**
	 * {@inheritDoc}
	 */
	public long position(byte[] bytes, long pos) throws SQLException {
		throw excep();
	}

	/**
	 * {@inheritDoc}
	 */
	public OutputStream setBinaryStream(long pos) throws SQLException {
		throw excep();
	}

	/**
	 * {@inheritDoc}
	 */
	public long position(Blob blob, long pos) throws SQLException {
		throw excep();
	}

	private static UnsupportedOperationException excep() {
		throw new UnsupportedOperationException("Blob may not be manipulated from creating session");
	}
}
