/*
 * Copyright (c) 2009 by Red Hat Inc and/or its affiliates or by
 * third-party contributors as indicated by either @author tags or express
 * copyright attribution statements applied by the authors.  All
 * third-party contributions are distributed under license by Red Hat Inc.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution; if not, write to:
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301  USA\
 */
package org.hibernate.engine.jdbc.jdbc4;

import java.sql.ResultSet;

import org.hibernate.engine.jdbc.JdbcSupport;
import org.hibernate.engine.jdbc.LobCreator;
import org.hibernate.engine.jdbc.LobCreationContext;
import org.hibernate.engine.jdbc.ColumnNameCache;

/**
 * TODO : javadoc
 *
 * @author Steve Ebersole
 */
public class JdbcSupportImpl implements JdbcSupport {
	private final boolean driverImplementsLobCreationMethods;

	public JdbcSupportImpl(boolean driverImplementsLobCreationMethods) {
		this.driverImplementsLobCreationMethods = driverImplementsLobCreationMethods;
	}

	/**
	 * {@inheritDoc}
	 */
	public LobCreator getLobCreator() {
		return NonContextualLobCreator.INSTANCE;
	}

	/**
	 * {@inheritDoc}
	 */
	public LobCreator getLobCreator(LobCreationContext lobCreationContext) {
		return driverImplementsLobCreationMethods
				? new ContextualLobCreator( lobCreationContext )
				: NonContextualLobCreator.INSTANCE;
	}

	/**
	 * {@inheritDoc}
	 */
	public ResultSet wrap(ResultSet resultSet, ColumnNameCache columnNameCache) {
		return new ResultSetWrapperImpl( resultSet, columnNameCache );
	}
}
