/*
 * Copyright (c) 2009 by Red Hat Inc and/or its affiliates or by
 * third-party contributors as indicated by either @author tags or express
 * copyright attribution statements applied by the authors.  All
 * third-party contributions are distributed under license by Red Hat Inc.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution; if not, write to:
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301  USA\
 */
package org.hibernate.engine.jdbc.jdbc4;

import java.sql.ResultSet;
import java.sql.RowId;
import java.sql.SQLException;
import java.sql.NClob;
import java.sql.SQLXML;
import java.io.Reader;
import java.io.InputStream;

import org.hibernate.engine.jdbc.ColumnNameCache;
import org.hibernate.engine.jdbc.ResultSetWrapper;

/**
 * TODO : javadoc
 *
 * @author Steve Ebersole
 */
public class ResultSetWrapperImpl extends org.hibernate.engine.jdbc.jdbc3.ResultSetWrapperImpl implements ResultSet {
	public ResultSetWrapperImpl(ResultSet resultSet, ColumnNameCache columnNameCache) {
		super( resultSet, columnNameCache );
	}

	public ResultSet getTarget() {
		return super.getTarget();
	}

	public int getHoldability() throws SQLException {
		return getTarget().getHoldability();
	}

	public boolean isClosed() throws SQLException {
		return getTarget().isClosed();
	}


	// SQLXML  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public SQLXML getSQLXML(int columnIndex) throws SQLException {
		return getTarget().getSQLXML( columnIndex );
	}

	public SQLXML getSQLXML(String columnLabel) throws SQLException {
		return getTarget().getSQLXML( findColumn( columnLabel ) );
	}

	public void updateSQLXML(int columnIndex, SQLXML xmlObject) throws SQLException {
		getTarget().updateSQLXML( columnIndex, xmlObject );
	}

	public void updateSQLXML(String columnLabel, SQLXML xmlObject) throws SQLException {
		getTarget().updateSQLXML( findColumn( columnLabel ), xmlObject );
	}


	// ROWID ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public RowId getRowId(int columnIndex) throws SQLException {
		return getTarget().getRowId( columnIndex );
	}

	public RowId getRowId(String columnLabel) throws SQLException {
		return getTarget().getRowId( findColumn( columnLabel ) );
	}

	public void updateRowId(int columnIndex, RowId rowId) throws SQLException {
		getTarget().updateRowId( columnIndex, rowId );
	}

	public void updateRowId(String columnLabel, RowId rowId) throws SQLException {
		getTarget().updateRowId( findColumn( columnLabel ), rowId );
	}


	// BLOB ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public void updateBlob(int columnIndex, InputStream inputStream, long length) throws SQLException {
		getTarget().updateBlob( columnIndex, inputStream, length );
	}

	public void updateBlob(String columnLabel, InputStream inputStream, long length) throws SQLException {
		getTarget().updateBlob( findColumn( columnLabel ), inputStream, length );
	}

	public void updateBlob(int columnIndex, InputStream inputStream) throws SQLException {
		getTarget().updateBlob( columnIndex, inputStream );
	}

	public void updateBlob(String columnLabel, InputStream inputStream) throws SQLException {
		getTarget().updateBlob( findColumn( columnLabel ), inputStream );
	}


	// CLOB ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public void updateClob(int columnIndex, Reader reader, long length) throws SQLException {
		getTarget().updateClob( columnIndex, reader, length );
	}

	public void updateClob(String columnLabel, Reader reader, long length) throws SQLException {
		getTarget().updateClob( findColumn( columnLabel ), reader, length );
	}

	public void updateClob(int columnIndex, Reader reader) throws SQLException {
		getTarget().updateClob( columnIndex, reader );
	}

	public void updateClob(String columnLabel, Reader reader) throws SQLException {
		getTarget().updateClob( findColumn( columnLabel ), reader );
	}


	// NCLOB  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public NClob getNClob(int columnIndex) throws SQLException {
		return getTarget().getNClob( columnIndex );
	}

	public NClob getNClob(String columnLabel) throws SQLException {
		return getTarget().getNClob( findColumn( columnLabel ) );
	}

	public void updateNClob(int columnIndex, Reader reader, long length) throws SQLException {
		getTarget().updateNClob( columnIndex, reader, length );
	}

	public void updateNClob(String columnLabel, Reader reader, long length) throws SQLException {
		getTarget().updateNClob( findColumn( columnLabel ), reader, length );
	}

	public void updateNClob(int columnIndex, NClob nClob) throws SQLException {
		getTarget().updateNClob( columnIndex, nClob );
	}

	public void updateNClob(String columnLabel, NClob nClob) throws SQLException {
		getTarget().updateNClob( findColumn( columnLabel ), nClob );
	}

	public void updateNClob(int columnIndex, Reader reader) throws SQLException {
		getTarget().updateNClob( columnIndex, reader );
	}

	public void updateNClob(String columnLabel, Reader reader) throws SQLException {
		getTarget().updateNClob( findColumn( columnLabel ), reader );
	}


	// NSTRING  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public String getNString(int columnIndex) throws SQLException {
		return getTarget().getNString( columnIndex );
	}

	public String getNString(String columnLabel) throws SQLException {
		return getTarget().getNString( findColumn( columnLabel ) );
	}

	public void updateNString(int columnIndex, String nString) throws SQLException {
		getTarget().updateNString( columnIndex, nString );
	}

	public void updateNString(String columnLabel, String nString) throws SQLException {
		getTarget().updateNString( findColumn( columnLabel ), nString );
	}


	// CharacterStream ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public void updateCharacterStream(int columnIndex, Reader reader, long length) throws SQLException {
		getTarget().updateCharacterStream( columnIndex, reader, length );
	}

	public void updateCharacterStream(String columnLabel, Reader reader, long length) throws SQLException {
		getTarget().updateCharacterStream( findColumn( columnLabel ), reader, length );
	}

	public void updateCharacterStream(int columnIndex, Reader reader) throws SQLException {
		getTarget().updateCharacterStream( columnIndex, reader );
	}

	public void updateCharacterStream(String columnLabel, Reader reader) throws SQLException {
		getTarget().updateCharacterStream( findColumn( columnLabel ), reader );
	}


	// NCharacterStream  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public Reader getNCharacterStream(int columnIndex) throws SQLException {
		return getTarget().getNCharacterStream( columnIndex );
	}

	public Reader getNCharacterStream(String columnLabel) throws SQLException {
		return getTarget().getNCharacterStream( findColumn( columnLabel ) );
	}

	public void updateNCharacterStream(int columnIndex, Reader reader, long length) throws SQLException {
		getTarget().updateNCharacterStream( columnIndex, reader, length );
	}

	public void updateNCharacterStream(String columnLabel, Reader reader, long length) throws SQLException {
		getTarget().updateNCharacterStream( findColumn( columnLabel ), reader, length );
	}

	public void updateNCharacterStream(int columnIndex, Reader reader) throws SQLException {
		getTarget().updateNCharacterStream( columnIndex, reader );
	}

	public void updateNCharacterStream(String columnLabel, Reader reader) throws SQLException {
		getTarget().updateNCharacterStream( findColumn( columnLabel ), reader );
	}


	// ASCIIStream  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public void updateAsciiStream(int columnIndex, InputStream stream, long length) throws SQLException {
		getTarget().updateAsciiStream( columnIndex, stream, length );
	}

	public void updateAsciiStream(String columnLabel, InputStream stream, long length) throws SQLException {
		getTarget().updateAsciiStream( findColumn( columnLabel ), stream, length );
	}

	public void updateAsciiStream(int columnIndex, InputStream stream) throws SQLException {
		getTarget().updateAsciiStream( columnIndex, stream );
	}

	public void updateAsciiStream(String columnLabel, InputStream stream) throws SQLException {
		getTarget().updateAsciiStream( findColumn( columnLabel ), stream );
	}


	// BinaryStream ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	public void updateBinaryStream(int columnIndex, InputStream stream, long length) throws SQLException {
		getTarget().updateBinaryStream( columnIndex, stream, length );
	}

	public void updateBinaryStream(String columnLabel, InputStream stream, long length) throws SQLException {
		getTarget().updateBinaryStream( findColumn( columnLabel ), stream, length );
	}

	public void updateBinaryStream(int columnIndex, InputStream x) throws SQLException {
	}

	public void updateBinaryStream(String columnLabel, InputStream x) throws SQLException {
	}


	// Wrapper Impl ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	@SuppressWarnings({ "unchecked" })
	public <T> T unwrap(Class<T> iface) throws SQLException {
		if ( ResultSetWrapper.class.isAssignableFrom( iface ) ) {
			return (T) this;
		}
		else if ( ResultSetWrapperImpl.class.isAssignableFrom( iface ) ) {
			return (T) this;
		}
		else {
			return getTarget().unwrap( iface );
		}
	}

	public boolean isWrapperFor(Class<?> iface) throws SQLException {
		return ResultSetWrapper.class.isAssignableFrom( iface )
				|| ResultSetWrapper.class.isAssignableFrom( iface );
	}
}
